#include <stdio.h>
#include <stdlib.h>

/* uncomment if you want to save the image */
//#define SAVE_IMAGE   /* encoding + saving to disk needs a LOT of time */

/* From savepng.c */
extern void savepng(char *fname, int *array, int width, int height);


void mandel(int width, int height, int *array, int maxiters) 
{
	double scale_real, scale_imag;
	double x, y, u, v, u2, v2;
	int 	 i, j, iter;

	scale_real = 3.5 / (double)width;
	scale_imag = 3.5 / (double)height;

	for (i = 0; i < height; i++) 
	{
		y = ((double)i * scale_imag) - 1.75;
		for (j = 0; j < width; j++) 
		{
			x = ((double)j * scale_real) - 2.25; 
			/* Optimized escape time algorithm */
			for (iter=0, u=v=u2=v2=0.0; (u2+v2 < 4.0 && iter < maxiters); iter++)
			{
				v = 2 * v * u + y;
				u = u2 - v2 + x;
				u2 = u*u;
				v2 = v*v;
			}
			if (iter == maxiters)
				iter = 0;		

			array[i*width + j] = iter;    
		}
	}
}


/* Usage: ./mandelbrot height maxiters */
int main(int argc, char *argv[]) 
{
	int n, width = 2600, height = 2600, maxiters = 400; /* default values */
	int *image;

	/* Arguments */
	if (argc >= 2) width = height = atoi(argv[1]);
	if (argc >= 3) maxiters = atoi(argv[2]);
	if (width < 10 || maxiters < 10)
		return 1;

	if ((image = (int *) malloc(width*height*sizeof(int))) == NULL)
	{
		fprintf(stderr, "memory allocation failed.\n");
		return 1;
	}

	mandel(width, height, image, maxiters);
	
#ifdef SAVE_IMAGE
	savepng("mandelbrot_s.png", image, width, height);
#endif

	return(0);
}
